/*
 * THIS CODE IS PROVIDED AS-IS WITH NO REPRESENTATIONS OR WARRANTIES, 
 * EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NONINFRINGEMENT. YOU ASSUME COMPLETE RESPONSIBILITY AND RISK 
 * FOR USE OF THIS CODE AND ANY AND ALL SITE-RELATED SERVICES. 
 */
package com.jnbridge.adapters.oaq;

import java.util.Hashtable;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.NamingException;
import java.net.URI;

public class InitialContextFactory implements
		javax.naming.spi.InitialContextFactory 
{
	public static final String oracleUrl = "jdbc:oracle:thin:";
	public static final String jnbridgeDBNameProperty = "com.jnbridge.adapters.oaq.dbname";

	public Context getInitialContext(Hashtable<?, ?> environment)
			throws NamingException {
		
		String dbURL = this.getDBURL(environment);
		String hostname = this.getHostname(environment);
		int port = this.getPort(environment);
		String sid = this.getSID();
	
		String username = this.getUsername(environment);
		String password = this.getPassword(environment);
		//throw new NamingException("DB URL= " + dbURL + ", Username= " + username + ", Password= " + password);
		OAQContext oaqCtx = null;
		
		try
		{
			oaqCtx = new OAQContext(hostname, username, password, sid, port);
		}
		catch (Exception ex)
		{
			throw new NamingException("Unable to create OAQContext: " + ex.getMessage());
		}	
			
		return oaqCtx;
	}
	
	private String getUsername(Hashtable<?, ?> environment) throws NamingException
	{
		
		String username = (String)environment.get(Context.SECURITY_PRINCIPAL);
		if (username == null || "".equals(username))
		{
			throw new NamingException("The property 'Context.SECURITY_PRINCIPAL' is null");
		}
		return username;
	}
	
	private String getPassword(Hashtable<?, ?> environment) throws NamingException
	{	
		String password = (String)environment.get(Context.SECURITY_CREDENTIALS);
		if (password == null || "".equals(password))
		{
			throw new NamingException("The property 'Context.SECURITY_CREDENTIALS' is null");
		}	
		return password;
	}
	
	private String getHostname(Hashtable<?, ?> environment) throws NamingException
	{
		String rtn = "";
		String aValue;
		URI aURL = null;
		

		aValue = (String)environment.get(Context.PROVIDER_URL);
		if (aValue == null || "".equals(aValue))
		{
			throw new NamingException("Property 'Context.PROVIDER_URL' not set");
		}
		
		try
		{
			aURL = new URI(aValue);
		}
		catch (Exception ex)
		{
			throw new NamingException("Malformed URL, " + aValue + ", error: " + ex.getMessage());
		}
		
		return aURL.getHost();
	}
	
	private int getPort(Hashtable<?, ?> environment) throws NamingException
	{
		String rtn = "";
		String aValue;
		URI aURL = null;
		

		aValue = (String)environment.get(Context.PROVIDER_URL);
		if (aValue == null || "".equals(aValue))
		{
			throw new NamingException("Property 'Context.PROVIDER_URL' not set");
		}
		
		try
		{
			aURL = new URI(aValue);
		}
		catch (Exception ex)
		{
			throw new NamingException("Malformed URL, " + aValue + ", error: " + ex.getMessage());
		}
		
		return aURL.getPort();
	}
	
	private String getSID() throws NamingException
	{
		String rtn = "";
		String aValue;
		
		try
		{
			aValue = System.getProperty(jnbridgeDBNameProperty);
			if ( aValue == null || "".equals(aValue))
			{
				throw new NamingException("No JVM argument: -Dcom.jnbridge.adapters.oaq.dbname=");
			}
		}
		catch (Exception ex)
		{
			throw new NamingException("Oracle DB name error: " + ex.getMessage());
		}
		
		rtn = aValue;	
		return rtn;
	}
	
	private String getDBURL(Hashtable<?, ?> environment) throws NamingException
	{
		String rtn = "";
		String aValue;
		URI aURL = null;
		

		aValue = (String)environment.get(Context.PROVIDER_URL);
		if (aValue == null || "".equals(aValue))
		{
			throw new NamingException("Property 'Context.PROVIDER_URL' not set");
		}
		
		try
		{
			aURL = new URI(aValue);
		}
		catch (Exception ex)
		{
			throw new NamingException("Malformed URL, " + aValue + ", error: " + ex.getMessage());
		}
		
		try
		{
			aValue = System.getProperty(jnbridgeDBNameProperty);
			if ( aValue == null || "".equals(aValue))
			{
				throw new NamingException("No JVM argument: -Dcom.jnbridge.adapters.oaq.dbname=");
			}
		}
		catch (Exception ex)
		{
			throw new NamingException("Oracle DB name error: " + ex.getMessage());
		}
		
		rtn = oracleUrl + "@" + aURL.getHost() + ":" + aURL.getPort() + ":" + aValue;	
		return rtn;
	}
}
